@@ -24,10 +24,6 @@
 
 #include "mdadm.h"
 
-#define REGISTER_DEV		_IO (MD_MAJOR, 1)
-#define START_MD		_IO (MD_MAJOR, 2)
-#define STOP_MD			_IO (MD_MAJOR, 3)
-
 int Build(char *mddev, struct mddev_dev *devlist,
 	  struct shape *s, struct context *c)
 {
@@ -39,14 +35,9 @@ int Build(char *mddev, struct mddev_dev *devlist,
 	 * geometry is 0xpp00cc
 	 * where pp is personality: 1==linear, 2=raid0
 	 * cc = chunk size factor: 0==4k, 1==8k etc.
-	 *
-	 * For md_version >= 0.90.0 we call
-	 * SET_ARRAY_INFO,  ADD_NEW_DISK, RUN_ARRAY
-	 *
 	 */
 	int i;
-	int vers;
-	struct stat stb;
+	dev_t rdev;
 	int subdevs = 0, missing_disks = 0;
 	struct mddev_dev *dv;
 	int bitmap_fd;
@@ -55,7 +46,13 @@ int Build(char *mddev, struct mddev_dev *devlist,
 	char chosen_name[1024];
 	int uuid[4] = {0,0,0,0};
 	struct map_ent *map = NULL;
+	mdu_array_info_t array;
+	mdu_param_t param; /* not used by syscall */
 
+	if (s->level == UnSet) {
+		pr_err("a RAID level is needed to Build an array.\n");
+		return 1;
+	}
 	/* scan all devices, make sure they really are block devices */
 	for (dv = devlist; dv; dv=dv->next) {
 		subdevs++;
@@ -63,16 +60,8 @@ int Build(char *mddev, struct mddev_dev *devlist,
 			missing_disks++;
 			continue;
 		}
-		if (stat(dv->devname, &stb)) {
-			pr_err("Cannot find %s: %s\n",
-				dv->devname, strerror(errno));
+		if (!stat_is_blkdev(dv->devname, NULL))
 			return 1;
-		}
-		if ((stb.st_mode & S_IFMT) != S_IFBLK) {
-			pr_err("%s is not a block device.\n",
-				dv->devname);
-			return 1;
-		}
 	}
 
 	if (s->raiddisks != subdevs) {
@@ -108,7 +97,7 @@ int Build(char *mddev, struct mddev_dev *devlist,
 	/* We need to create the device.  It can have no name. */
 	map_lock(&map);
 	mdfd = create_mddev(mddev, NULL, c->autof, LOCAL,
-			    chosen_name);
+			    chosen_name, 0);
 	if (mdfd < 0) {
 		map_unlock(&map);
 		return 1;
@@ -118,39 +107,30 @@ int Build(char *mddev, struct mddev_dev *devlist,
 	map_update(&map, fd2devnm(mdfd), "none", uuid, chosen_name);
 	map_unlock(&map);
 
-	vers = md_get_version(mdfd);
-
-	/* looks Ok, go for it */
-	if (vers >= 9000) {
-		mdu_array_info_t array;
-		array.level = s->level;
-		if (s->size == MAX_SIZE)
-			s->size = 0;
-		array.size = s->size;
-		array.nr_disks = s->raiddisks;
-		array.raid_disks = s->raiddisks;
-		array.md_minor = 0;
-		if (fstat(mdfd, &stb)==0)
-			array.md_minor = minor(stb.st_rdev);
-		array.not_persistent = 1;
-		array.state = 0; /* not clean, but no errors */
-		if (s->assume_clean)
-			array.state |= 1;
-		array.active_disks = s->raiddisks - missing_disks;
-		array.working_disks = s->raiddisks - missing_disks;
-		array.spare_disks = 0;
-		array.failed_disks = missing_disks;
-		if (s->chunk == 0 && (s->level==0 || s->level==LEVEL_LINEAR))
-			s->chunk = 64;
-		array.chunk_size = s->chunk*1024;
-		array.layout = s->layout;
-		if (ioctl(mdfd, SET_ARRAY_INFO, &array)) {
-			pr_err("SET_ARRAY_INFO failed for %s: %s\n",
-				mddev, strerror(errno));
-			goto abort;
-		}
-	} else if (s->bitmap_file) {
-		pr_err("bitmaps not supported with this kernel\n");
+	array.level = s->level;
+	if (s->size == MAX_SIZE)
+		s->size = 0;
+	array.size = s->size;
+	array.nr_disks = s->raiddisks;
+	array.raid_disks = s->raiddisks;
+	array.md_minor = 0;
+	if (fstat_is_blkdev(mdfd, mddev, &rdev))
+		array.md_minor = minor(rdev);
+	array.not_persistent = 1;
+	array.state = 0; /* not clean, but no errors */
+	if (s->assume_clean)
+		array.state |= 1;
+	array.active_disks = s->raiddisks - missing_disks;
+	array.working_disks = s->raiddisks - missing_disks;
+	array.spare_disks = 0;
+	array.failed_disks = missing_disks;
+	if (s->chunk == 0 && (s->level==0 || s->level==LEVEL_LINEAR))
+		s->chunk = 64;
+	array.chunk_size = s->chunk*1024;
+	array.layout = s->layout;
+	if (md_set_array_info(mdfd, &array)) {
+		pr_err("md_set_array_info() failed for %s: %s\n",
+		       mddev, strerror(errno));
 		goto abort;
 	}
 
@@ -163,20 +143,14 @@ int Build(char *mddev, struct mddev_dev *devlist,
 	}
 	/* now add the devices */
 	for ((i=0), (dv = devlist) ; dv ; i++, dv=dv->next) {
+		mdu_disk_info_t disk;
 		unsigned long long dsize;
 		int fd;
+
 		if (strcmp("missing", dv->devname) == 0)
 			continue;
-		if (stat(dv->devname, &stb)) {
-			pr_err("Weird: %s has disappeared.\n",
-				dv->devname);
+		if (!stat_is_blkdev(dv->devname, &rdev))
 			goto abort;
-		}
-		if ((stb.st_mode & S_IFMT)!= S_IFBLK) {
-			pr_err("Weird: %s is no longer a block device.\n",
-				dv->devname);
-			goto abort;
-		}
 		fd = open(dv->devname, O_RDONLY|O_EXCL);
 		if (fd < 0) {
 			pr_err("Cannot open %s: %s\n",
@@ -187,94 +161,58 @@ int Build(char *mddev, struct mddev_dev *devlist,
 		    (s->size == 0 || s->size == MAX_SIZE || dsize < s->size))
 				s->size = dsize;
 		close(fd);
-		if (vers >= 9000) {
-			mdu_disk_info_t disk;
-			disk.number = i;
-			disk.raid_disk = i;
-			disk.state = (1<<MD_DISK_SYNC) | (1<<MD_DISK_ACTIVE);
-			if (dv->writemostly == 1)
-				disk.state |= 1<<MD_DISK_WRITEMOSTLY;
-			disk.major = major(stb.st_rdev);
-			disk.minor = minor(stb.st_rdev);
-			if (ioctl(mdfd, ADD_NEW_DISK, &disk)) {
-				pr_err("ADD_NEW_DISK failed for %s: %s\n",
-					dv->devname, strerror(errno));
-				goto abort;
-			}
-		} else {
-			if (ioctl(mdfd, REGISTER_DEV, &stb.st_rdev)) {
-				pr_err("REGISTER_DEV failed for %s: %s.\n",
-					dv->devname, strerror(errno));
-				goto abort;
-			}
+		disk.number = i;
+		disk.raid_disk = i;
+		disk.state = (1<<MD_DISK_SYNC) | (1<<MD_DISK_ACTIVE);
+		if (dv->writemostly == FlagSet)
+			disk.state |= 1<<MD_DISK_WRITEMOSTLY;
+		disk.major = major(rdev);
+		disk.minor = minor(rdev);
+		if (ioctl(mdfd, ADD_NEW_DISK, &disk)) {
+			pr_err("ADD_NEW_DISK failed for %s: %s\n",
+			       dv->devname, strerror(errno));
+			goto abort;
 		}
 	}
 	/* now to start it */
-	if (vers >= 9000) {
-		mdu_param_t param; /* not used by syscall */
-		if (s->bitmap_file) {
+	if (s->bitmap_file) {
+		bitmap_fd = open(s->bitmap_file, O_RDWR);
+		if (bitmap_fd < 0) {
+			int major = BITMAP_MAJOR_HI;
+#if 0
+			if (s->bitmap_chunk == UnSet) {
+				pr_err("%s cannot be opened.\n", s->bitmap_file);
+				goto abort;
+			}
+#endif
+			bitmapsize = s->size >> 9; /* FIXME wrong for RAID10 */
+			if (CreateBitmap(s->bitmap_file, 1, NULL,
+					 s->bitmap_chunk, c->delay,
+					 s->write_behind, bitmapsize, major)) {
+				goto abort;
+			}
 			bitmap_fd = open(s->bitmap_file, O_RDWR);
 			if (bitmap_fd < 0) {
-				int major = BITMAP_MAJOR_HI;
-#if 0
-				if (s->bitmap_chunk == UnSet) {
-					pr_err("%s cannot be openned.",
-						s->bitmap_file);
-					goto abort;
-				}
-#endif
-				if (vers < 9003) {
-					major = BITMAP_MAJOR_HOSTENDIAN;
-#ifdef __BIG_ENDIAN
-					pr_err("Warning - bitmaps created on this kernel are not portable\n"
-						"  between different architectures.  Consider upgrading the Linux kernel.\n");
-#endif
-				}
-				bitmapsize = s->size>>9; /* FIXME wrong for RAID10 */
-				if (CreateBitmap(s->bitmap_file, 1, NULL, s->bitmap_chunk,
-						 c->delay, s->write_behind, bitmapsize, major)) {
-					goto abort;
-				}
-				bitmap_fd = open(s->bitmap_file, O_RDWR);
-				if (bitmap_fd < 0) {
-					pr_err("%s cannot be openned.",
-						s->bitmap_file);
-					goto abort;
-				}
-			}
-			if (bitmap_fd >= 0) {
-				if (ioctl(mdfd, SET_BITMAP_FILE, bitmap_fd) < 0) {
-					pr_err("Cannot set bitmap file for %s: %s\n",
-						mddev, strerror(errno));
-					goto abort;
-				}
+				pr_err("%s cannot be opened.\n", s->bitmap_file);
+				goto abort;
 			}
 		}
-		if (ioctl(mdfd, RUN_ARRAY, &param)) {
-			pr_err("RUN_ARRAY failed: %s\n",
-				strerror(errno));
-			if (s->chunk & (s->chunk-1)) {
-				cont_err("Problem may be that chunk size is not a power of 2\n");
+		if (bitmap_fd >= 0) {
+			if (ioctl(mdfd, SET_BITMAP_FILE, bitmap_fd) < 0) {
+				pr_err("Cannot set bitmap file for %s: %s\n",
+				       mddev, strerror(errno));
+				goto abort;
 			}
-			goto abort;
-		}
-	} else {
-		unsigned long arg;
-		arg=0;
-		while (s->chunk > 4096) {
-			arg++;
-			s->chunk >>= 1;
-		}
-		if (s->level == 0)
-			arg |= 0x20000;
-		else
-			arg |= 0x10000;
-		if (ioctl(mdfd, START_MD, arg)) {
-			pr_err("START_MD failed: %s\n",
-				strerror(errno));
-			goto abort;
 		}
 	}
+	if (ioctl(mdfd, RUN_ARRAY, &param)) {
+		pr_err("RUN_ARRAY failed: %s\n", strerror(errno));
+		if (s->chunk & (s->chunk - 1)) {
+			cont_err("Problem may be that chunk size is not a power of 2\n");
+		}
+		goto abort;
+	}
+
 	if (c->verbose >= 0)
 		pr_err("array %s built and started.\n",
 			mddev);
@@ -283,10 +221,7 @@ int Build(char *mddev, struct mddev_dev *devlist,
 	return 0;
 
  abort:
-	if (vers >= 9000)
-	    ioctl(mdfd, STOP_ARRAY, 0);
-	else
-	    ioctl(mdfd, STOP_MD, 0);
+	ioctl(mdfd, STOP_ARRAY, 0);
 	close(mdfd);
 	return 1;
 }
